/*
###############################################################################
# Copyright (c) 2017, PulseRain Technology LLC 
#
# This program is distributed under a dual license: an open source license, 
# and a commercial license. 
# 
# The open source license under which this program is distributed is the 
# GNU Public License version 3 (GPLv3).
#
# And for those who want to use this program in ways that are incompatible
# with the GPLv3, PulseRain Technology LLC offers commercial license instead.
# Please contact PulseRain Technology LLC (www.pulserain.com) for more detail.
#
###############################################################################
*/


//`include "encoder.vh"

//`default_nettype none

module encoder_core  (
    //=======================================================================
    // clock / reset
    //=======================================================================
        
    input   wire                                clk, //100MHz
    input   wire                                reset_n,
    
    //=======================================================================
    // 
    //=======================================================================
    
    input   wire                                encoder_L_A,
	input   wire                                encoder_L_B,
    input   wire                                encoder_R_A,
	input   wire                                encoder_R_B,
    output  wire           [15 : 0]             counter_L,
    output  wire           [15 : 0]             counter_R
    
);


	//parameter N = 2000000;
    logic  clk_100 ;
	logic  clk_1M ;
	logic  [1:0] datatemp_L_A_UP;
	logic  [1:0] datatemp_L_B_UP;

	
	logic  [1:0] datatemp_R_A_UP;
	logic  [1:0] datatemp_R_B_UP;
	
	logic  [31:0] cnt_clk;
	logic  [31:0] cnt_clk_1M;
	
	logic  [15:0] cnt_temp_L_A_UP;
	logic  [15:0] cnt_temp_L_B_UP;

	logic  [15:0] cnt_temp_R_A_UP;
	logic  [15:0] cnt_temp_R_B_UP;
	
	
	logic  [15 : 0]              counter_L_A_UP;
	logic  [15 : 0]              counter_L_B_UP;

	logic  [15 : 0]              counter_R_A_UP;
	logic  [15 : 0]              counter_R_B_UP;

  //logic en1 = 1;
  //logic en2 = 1;
  
  assign counter_L = counter_L_A_UP; //+counter_L_B_UP;
  assign counter_R = counter_R_A_UP; //+counter_R_B_UP;
  
  
  
//将100MHz的clk分频为1MHZ的clk_1M
always_ff@(posedge clk, negedge reset_n)
begin : CLK_1M_DIVIDE
    if(~reset_n) begin
      cnt_clk_1M <= 0;
    end else if(cnt_clk_1M >= 32'd99)begin
		cnt_clk_1M <= 0;
	end else begin
		cnt_clk_1M <= cnt_clk_1M + 1'b1;
	end
end : CLK_1M_DIVIDE

always_ff@(posedge clk, negedge reset_n)
begin : CLK_1M_GEN
    if(~reset_n) begin
      clk_1M <= 1'b0;
    end else if(cnt_clk_1M <= 32'd50-1)begin
		clk_1M <= 1'b0;
	end else begin
		clk_1M <= 1'b1;
	end
end : CLK_1M_GEN


////////////////////////////////////对encoder_A消抖/////////////////////////////////////////
logic	encoder_A1,encoder_A2,encoder_A3,encoder_A4,encoder_A5,encoder_A6,encoder_A7,encoder_A8;
//消除亚稳态
always_ff@(posedge clk_1M) begin
	encoder_A1	<=	encoder_L_A;
	encoder_A2	<=	encoder_A1;
	encoder_A3	<=	encoder_A2;
	encoder_A4	<=	encoder_A3;
	encoder_A5	<=	encoder_A4;
	encoder_A6	<=	encoder_A5;
	encoder_A7	<=	encoder_A6;
	encoder_A8	<=	encoder_A7;
end 

logic	encoder_A_reg;
//简单去抖动处理
always_ff@(encoder_A1 or encoder_A2 or encoder_A3 or encoder_A4 or encoder_A5 or encoder_A6 or encoder_A7 or encoder_A8) begin
	case({encoder_A1,encoder_A2,encoder_A3,encoder_A4,encoder_A5,encoder_A6,encoder_A7,encoder_A8})
		8'b11111111:	encoder_A_reg <= 1'b1;
		8'b00000000:	encoder_A_reg <= 1'b0;
		default: encoder_A_reg <= encoder_A_reg;
	endcase
end

////////////////////////////////////对encoder_B消抖/////////////////////////////////////////
logic	encoder_B1,encoder_B2,encoder_B3,encoder_B4,encoder_B5,encoder_B6,encoder_B7,encoder_B8;
//消除亚稳态
always_ff@(posedge clk_1M) begin
	encoder_B1	<=	encoder_L_B;
	encoder_B2	<=	encoder_B1;
	encoder_B3	<=	encoder_B2;
	encoder_B4	<=	encoder_B3;
	encoder_B5	<=	encoder_B4;
	encoder_B6	<=	encoder_B5;
	encoder_B7	<=	encoder_B6;
	encoder_B8	<=	encoder_B7;
end 

logic	encoder_B_reg;
//简单去抖动处理
always_ff@(encoder_B1 or encoder_B2 or encoder_B3 or encoder_B4 or encoder_B5 or encoder_B6 or encoder_B7 or encoder_B8) begin
	case({encoder_B1,encoder_B2,encoder_B3,encoder_B4,encoder_B5,encoder_B6,encoder_B7,encoder_B8})
		8'b11111111:	encoder_B_reg <= 1'b1;
		8'b00000000:	encoder_B_reg <= 1'b0;
		default: encoder_B_reg <= encoder_B_reg;
	endcase
end

//将100MHz的clk分频为100HZ的clk_100
always_ff@(posedge clk, negedge reset_n)
begin : CLK_100_DIVIDE
    if(~reset_n) begin
      cnt_clk <= 0;
    end else if(cnt_clk >= 32'd4999999)begin
		cnt_clk <= 0;
	end else begin
		cnt_clk <= cnt_clk + 1'b1;
	end
end : CLK_100_DIVIDE

always_ff@(posedge clk, negedge reset_n)
begin : CLK_100_GEN
    if(~reset_n) begin
      clk_100 <= 1'b0;
    end else if(cnt_clk <= 32'd2500000-1)begin
		clk_100 <= 1'b0;
	end else begin
		clk_100 <= 1'b1;
	end
end : CLK_100_GEN
	 
	 
	
	////////////////检测Left轮子的上升沿和下降沿////////////////////////////
	
	//检测左轮A相的上升沿
	always_ff@ (posedge encoder_A_reg,negedge reset_n) 
	begin:clk1
	 if (!reset_n) 
	 begin
	   datatemp_L_A_UP <= 2'b0; 
	 end
	 else begin
	   datatemp_L_A_UP <= {clk_100,datatemp_L_A_UP[1]}; end
	end:clk1

	
	//检测轮B相的上升沿
	always_ff@ (posedge encoder_L_B,negedge reset_n) 
	begin:clk3
	 if (!reset_n) 
	 begin
	   datatemp_L_B_UP <= 2'b0; 
	 end
	 else begin
	   datatemp_L_B_UP <= {clk_100,datatemp_L_B_UP[1]}; end
	end:clk3
	

	////////////////////检测Right轮子的上升沿和下降沿////////////////////////////
	//检测右轮A相的上升沿
	always_ff@ (posedge encoder_B_reg,negedge reset_n)
	begin:clk5
	 if (!reset_n) 
	 begin
	   datatemp_R_A_UP <= 2'b0; 
	 end
	 else begin
	   datatemp_R_A_UP <= {clk_100,datatemp_R_A_UP[1]}; end
	end:clk5
	
	//检测右轮B相的上升沿
	always_ff@ (posedge encoder_R_B,negedge reset_n)
	begin:clk7
	 if (!reset_n) 
	 begin
	   datatemp_R_B_UP <= 2'b0; 
	 end
	 else begin
	   datatemp_R_B_UP <= {clk_100,datatemp_R_B_UP[1]}; end
	end:clk7
	
	
	
	
	////////////////////////////////记录左侧轮子10ms内的脉冲数////////////////////////////
	//10ms内左侧A相上升沿个数
	always_ff@(posedge encoder_A_reg,negedge reset_n)
	begin:clk9
	if (!reset_n) 
	      begin
	         counter_L_A_UP <= 16'b0; 
				cnt_temp_L_A_UP <= 16'b0;
			end 
	else  if(datatemp_L_A_UP == 2'b10)
			begin
				counter_L_A_UP <= cnt_temp_L_A_UP; //+ 16'b1;  //16'd1 ; cnt_temp1 + 16'b1
				cnt_temp_L_A_UP <= 16'b0;
			end
	else  begin
				cnt_temp_L_A_UP <= cnt_temp_L_A_UP + 16'b1; 
			end
    end:clk9
	
	//10ms内左侧B相上升沿个数
	always_ff@(posedge encoder_L_B,negedge reset_n)
	begin:clk11
	if (!reset_n) 
	      begin
	         counter_L_B_UP <= 16'b0; 
				cnt_temp_L_B_UP <= 16'b0;
			end 
	else  if(datatemp_L_B_UP == 2'b10)
			begin
				counter_L_B_UP <= cnt_temp_L_B_UP; //+ 16'b1;  //16'd1 ; cnt_temp1 + 16'b1
				cnt_temp_L_B_UP <= 16'b0;
			end
	else  begin
				cnt_temp_L_B_UP <= cnt_temp_L_B_UP + 16'b1; 
			end
    end:clk11
	
		
	////////////////////////////////记录右侧轮子10ms内的脉冲数////////////////////////////
	//10ms内右侧A相上升沿个数
	always_ff@(posedge encoder_B_reg,negedge reset_n)
	begin:clk13
	if (!reset_n) 
	      begin
	         counter_R_A_UP <= 16'b0; 
				cnt_temp_R_A_UP <= 16'b0;
			end 
	else  if(datatemp_R_A_UP == 2'b10)
			begin
				counter_R_A_UP <= cnt_temp_R_A_UP; //+ 16'b1;  //16'd1 ; cnt_temp1 + 16'b1
				cnt_temp_R_A_UP <= 16'b0;
			end
	else  begin
				cnt_temp_R_A_UP <= cnt_temp_R_A_UP + 16'b1; 
			end
    end:clk13
	
	//10ms内右侧B相上升沿个数
	always_ff@(posedge encoder_R_B,negedge reset_n)
	begin:clk15
	if (!reset_n) 
	      begin
	         counter_R_B_UP <= 16'b0; 
				cnt_temp_R_B_UP <= 16'b0;
			end 
	else  if(datatemp_R_B_UP == 2'b10)
			begin
				counter_R_B_UP <= cnt_temp_R_B_UP; //+ 16'b1;  //16'd1 ; cnt_temp1 + 16'b1
				cnt_temp_R_B_UP <= 16'b0;
			end
	else  begin
				cnt_temp_R_B_UP <= cnt_temp_R_B_UP + 16'b1; 
			end
    end:clk15
	
	
endmodule : encoder_core

//`default_nettype wire