/*
###############################################################################
# Copyright (c) 2017, PulseRain Technology LLC 
#
# This program is distributed under a dual license: an open source license, 
# and a commercial license. 
# 
# The open source license under which this program is distributed is the 
# GNU Public License version 3 (GPLv3).
#
# And for those who want to use this program in ways that are incompatible
# with the GPLv3, PulseRain Technology LLC offers commercial license instead.
# Please contact PulseRain Technology LLC (www.pulserain.com) for more detail.
#
###############################################################################
*/


//`include "encoder.vh"

//`default_nettype none

module encoder_core  (
    //=======================================================================
    // clock / reset
    //=======================================================================
        
    input   wire                                clk, //100MHz
    input   wire                                reset_n,
    
    //=======================================================================
    // 
    //=======================================================================
    
    input   wire                                encoder_A,
    input   wire                                encoder_B,
    output  wire           [15 : 0]              counter_A,
    output  wire           [15 : 0]              counter_B
    
);


	parameter N = 1000000;
   logic  clk_100;
	logic  [1:0] datatemp_A = 0;
	logic  [1:0] datatemp_B = 0;
	logic  [31:0] cnt_clk = 0;
	logic  [15:0] cnt_temp1 = 0;
	logic  [15:0] cnt_temp2 = 0;
	
  logic  [15 : 0]              counter_A_O;
  logic  [15 : 0]              counter_B_O;
  
  logic en1 = 1;
  logic en2 = 1;
  
  assign counter_A = counter_A_O;
  assign counter_B = counter_B_O;
	
	//将100MHz的clk分频为100HZ的clk_100

always_ff@(posedge clk, negedge reset_n)
begin : CLK_100_DIVIDE
    if(~reset_n) begin
      cnt_clk <= 0;
    end else if(cnt_clk >= 32'd99999)begin
		cnt_clk <= 0;
	end else begin
		cnt_clk <= cnt_clk + 1'b1;
	end
end : CLK_100_DIVIDE


always_ff@(posedge clk, negedge reset_n)
begin : CLK_100_GEN
    if(~rst) begin
      clk_100 <= 1'b0;
    end else if(count_1m <= 32'd500000-1)begin
		clk_100 <= 1'b0;
	end else begin
		clk_100 <= 1'b1;
	end
end : CLK_100_GEN
	 
	 
	
	//检测上升沿
	always_ff@ (posedge encoder_A,negedge reset_n) 
	begin:clk2
	 if (!reset_n)
	   datatemp_A = 2'b0;
	 else
	   datatemp_A = {clk_100,datatemp_A[1]};
	end:clk2
	
	
	always_ff@ (posedge encoder_B,negedge reset_n)
	begin:clk3
	 if (!reset_n)
	   datatemp_B = 2'b0;
	 else
	   datatemp_B = {clk_100,datatemp_B[1]};
	end:clk3
	
	
	
	//两次上升沿中间计数A
	always_ff@(posedge encoder_A,negedge reset_n)
	begin:clk4
	 if (!reset_n) 
	        begin
	            counter_A_O = 16'b0; 
				cnt_temp1 <= 16'b0;
			end 
	else if(en1)
			if(datatemp_A == 2'b10)
				begin
					counter_A_O <= 16'b1;  //16'd1 ; cnt_temp1 + 16'b1
					cnt_temp1 <= 16'b0;
				end
			else 
				begin
					cnt_temp1 <= cnt_temp1 + 16'b1; 
				end
	else
		        begin
	            counter_A_O = 16'b0; 
				cnt_temp1 <= 16'b0;
			    end 
	
    end:clk4
		
	//两次上升沿中间计数B
	always_ff@(posedge encoder_B,negedge reset_n)
	begin:clk5
	 if (!reset_n) 
	        begin
	            counter_B_O = 16'b0; 
				cnt_temp2 <= 16'b0;
			end 
	else if(en2)
			if(datatemp_B == 2'b10)
				begin
					counter_B_O <= 16'b1;  //16'd1 ; cnt_temp2 + 16'b1
					cnt_temp2 <= 16'b0;
				end
			else 
				begin
					cnt_temp2 <= cnt_temp2 + 16'b1; 
				end
	else
		        begin
	            counter_B_O = 16'b0; 
				cnt_temp2 <= 16'b0;
			    end 
    end:clk5
	
	
endmodule : encoder_core

//`default_nettype wire