/*
###############################################################################
# Copyright (c) 2017, PulseRain Technology LLC 
#
# This program is distributed under a dual license: an open source license, 
# and a commercial license. 
# 
# The open source license under which this program is distributed is the 
# GNU Public License version 3 (GPLv3).
#
# And for those who want to use this program in ways that are incompatible
# with the GPLv3, PulseRain Technology LLC offers commercial license instead.
# Please contact PulseRain Technology LLC (www.pulserain.com) for more detail.
#
###############################################################################
*/


//`include "encoder.vh"

`default_nettype none

module encoder_core  (
    //=======================================================================
    // clock / reset
    //=======================================================================
        
    input   wire                                clk,
    input   wire                                reset_n,
    
    //=======================================================================
    // 
    //=======================================================================
    
    input   wire                                 encoder_A,
    input   wire                                 encoder_B,
    output  wire unsigned [15 : 0]               counter_A,
    output  wire unsigned [15 : 0]               counter_B
    
);

    localparam				                    NUM_10ms	=	100000;
	 logic				                          [16:0]	cnt = 0;
    logic [15:0] counter_A_0,counter_A_0_reg,counter_A_1,counter_A_1_reg,counter_A_2,counter_A_2_reg,counter_A_3,counter_A_3_reg,counter_A_4,counter_A_4_reg,counter_A_5,counter_A_5_reg,counter_A_6,counter_A_6_reg,counter_A_7,counter_A_7_reg,counter_A_8,counter_A_8_reg,counter_A_9,counter_A_9_reg,
         counter_B_0,counter_B_0_reg,counter_B_1,counter_B_1_reg,counter_B_2,counter_B_2_reg,counter_B_3,counter_B_3_reg,counter_B_4,counter_B_4_reg,counter_B_5,counter_B_5_reg,counter_B_6,counter_B_6_reg,counter_B_7,counter_B_7_reg,counter_B_8,counter_B_8_reg,counter_B_9,counter_B_9_reg;
    logic   encoder_A_reg ,encoder_B_reg ;
			
always_ff@(posedge clk or negedge reset_n)
	begin
	     if(!reset_n) 
		   cnt <=0;
	     else if(cnt >= NUM_10ms-1)   
		   cnt <= 1'b0;
         else 
		   cnt <= cnt + 1'b1;
    end

    wire      [1:0]   key;                            //输入的按键					
	wire      [1:0]   key_pulse;                      //按键动作产生的脉冲	

    logic     [1:0]   key_reset_n_pre;                  //定义一个寄存器型变量存储上一个触发时的按键值
    logic     [1:0]   key_reset_n;                      //定义一个寄存器变量储存储当前时刻触发的按键值
 
    wire      [1:0]   key_edge;                         //检测到按键由高到低变化是产生一个高脉冲
	
    assign    key[0] = encoder_A;
	assign    key[1] = encoder_B;
    
 always_ff @(posedge clk  or  negedge reset_n)
   begin
      if (!reset_n) begin
          key_reset_n <= 2'b00;                 //初始化时给key_reset_n赋值全为1，{}中表示N个1
          key_reset_n_pre <= 2'b00;
      end
      else begin
          key_reset_n <= key;                       //第一个时钟上升沿触发之后key的值赋给key_reset_n,同时key_reset_n的值赋给key_reset_n_pre
          key_reset_n_pre <= key_reset_n;             //非阻塞赋值。相当于经过两个时钟触发，key_reset_n存储的是当前时刻key的值，key_reset_n_pre存储的是前一个时钟的key的值
      end    
    end

 assign  key_edge = (!key_reset_n_pre) & (key_reset_n);//脉冲边沿检测。当key检测到下降沿时，key_edge产生一个时钟周期的高电平

 logic	 [17:0]	  cnt2;                            //产生延时所用的计数器，系统时钟12MHz，要延时20ms左右时间，至少需要18位计数器     


 always_ff @(posedge clk or negedge reset_n)
    begin
      if(!reset_n)
         cnt2 <= 18'd0;
      else if(key_edge)
         cnt2 <= 18'd0;
      else
         cnt2 <= cnt2 + 1'd1;
      end  

 logic     [1:0]   key_sec_pre;                    //延时后检测电平寄存器变量
 logic     [1:0]   key_sec;                    



 always_ff @(posedge clk  or  negedge reset_n)
   begin
      if (!reset_n) 
          key_sec <= 2'b00;                
      else if (cnt2==18'd500)
          key_sec <= key;  
   end
always @(posedge clk  or  negedge reset_n)
   begin
      if (!reset_n)
          key_sec_pre <= 2'b00;
      else                   
          key_sec_pre <= key_sec;             
  end      
assign  key_pulse =(!key_sec_pre) & (key_sec); 


always_ff @(posedge clk or negedge reset_n)
    begin
	if(!reset_n)
	begin
	     counter_A_0 <=0;counter_A_0_reg<=0;
		 counter_A_1 <=0;counter_A_1_reg<=0;
		 counter_A_2 <=0;counter_A_2_reg<=0;
		 counter_A_3 <=0;counter_A_3_reg<=0;
		 counter_A_4 <=0;counter_A_4_reg<=0;
		 counter_A_5 <=0;counter_A_5_reg<=0;
		 counter_A_6 <=0;counter_A_6_reg<=0;
		 counter_A_7 <=0;counter_A_7_reg<=0;
		 counter_A_8 <=0;counter_A_8_reg<=0;
		 counter_A_9 <=0;counter_A_9_reg<=0;
	end
	else if(cnt<17'd10000&&key_pulse[0])
	
	begin
		 counter_A_0 <= counter_A_0 + 1;
		 counter_A_0_reg <= counter_A_0;
		 counter_A_9 <= 0;
      
	 end
    else if(cnt<17'd20000 && cnt>=17'd10000&&key_pulse[0])

	begin
		 counter_A_1 <= counter_A_1 + 1;
		 counter_A_1_reg <= counter_A_1;
		 counter_A_0 <=0;
	end
    else if(cnt<17'd30000 && cnt>=17'd20000&&key_pulse[0])

	begin
		 counter_A_2 <= counter_A_2 + 1;
		 counter_A_2_reg <= counter_A_2;
		 counter_A_1 <=0;
	end
    else if(cnt<17'd40000 && cnt>=17'd30000&&key_pulse[0])

	begin
		 counter_A_3 <= counter_A_3 + 1;
		 counter_A_3_reg <= counter_A_3;
		 counter_A_2 <=0;
	end
    else if(cnt<17'd50000 && cnt>=17'd40000&&key_pulse[0])

	begin
		 counter_A_4 <= counter_A_4 + 1;
         counter_A_4_reg <= counter_A_4;
		 counter_A_3 <=0;
	end
    else if(cnt<17'd60000 && cnt>=17'd50000&&key_pulse[0])

	begin
		 counter_A_5 <= counter_A_5 + 1;
		 counter_A_5_reg <= counter_A_5;
		 counter_A_4 <=0;
	end
    else if(cnt<17'd70000 && cnt>=17'd60000&&key_pulse[0])

	begin
		 counter_A_6 <= counter_A_6 + 1;
         counter_A_6_reg <= counter_A_6;
		 counter_A_5 <=0;
	end
    else if(cnt<17'd80000 && cnt>=17'd70000&&key_pulse[0])

	begin
		 counter_A_7 <= counter_A_7 + 1;
		 counter_A_7_reg <= counter_A_7;
		 counter_A_6 <=0;
	end
    else if(cnt<17'd90000 && cnt>=17'd80000&&key_pulse[0])

	begin
		 counter_A_8 <= counter_A_8 + 1;
		 counter_A_8_reg <= counter_A_8;
		 counter_A_7 <=0;
	end
    else if(cnt<17'd100000 && cnt>=17'd90000&&key_pulse[0])

	begin
		 counter_A_9 <= counter_A_9 + 1;
		 counter_A_9_reg <= counter_A_9;
		 counter_A_8 <=0;	
    end		 
       
    end


always_ff @(posedge clk or negedge reset_n)
    begin
	if(!reset_n)
	begin
	     counter_B_0 <=0;counter_B_0_reg<=0;
		 counter_B_1 <=0;counter_B_1_reg<=0;
		 counter_B_2 <=0;counter_B_2_reg<=0;
		 counter_B_3 <=0;counter_B_3_reg<=0;
		 counter_B_4 <=0;counter_B_4_reg<=0;
		 counter_B_5 <=0;counter_B_5_reg<=0;
		 counter_B_6 <=0;counter_B_6_reg<=0;
		 counter_B_7 <=0;counter_B_7_reg<=0;
		 counter_B_8 <=0;counter_B_8_reg<=0;
		 counter_B_9 <=0;counter_B_9_reg<=0;
	end
	else if(cnt<17'd10000&&key_pulse[1])
	begin
		 counter_B_0 <= counter_B_0 + 1;
		 counter_B_0_reg <= counter_B_0;
		 counter_B_9 <= 0;
	end
    else if(cnt<17'd20000 && cnt >=17'd10000&&key_pulse[1])
	begin
		 counter_B_1 <= counter_B_1 + 1;
		 counter_B_1_reg <= counter_B_1;
		 counter_B_0 <=0;
	end
    else if(cnt<17'd30000 && cnt >=17'd20000&&key_pulse[1])
	begin
		 counter_B_2 <= counter_B_2 + 1;
		 counter_B_2_reg <= counter_B_2;
		 counter_B_1 <=0;
	end
    else if(cnt<17'd40000 && cnt>=17'd30000&&key_pulse[1] )
	begin
		 counter_B_3 <= counter_B_3 + 1;
		 counter_B_3_reg <= counter_B_3;
		 counter_B_2 <=0;
	end
    else if(cnt<17'd50000 && cnt>=17'd40000&&key_pulse[1] )
	begin
		 counter_B_4 <= counter_B_4 + 1;
         counter_B_4_reg <= counter_B_4;
		 counter_B_3 <=0;
	end
    else if(cnt<17'd60000 &&  cnt>=17'd50000&&key_pulse[1])
	begin
		 counter_B_5 <= counter_B_5 + 1;
		 counter_B_5_reg <= counter_B_5;
		 counter_B_4 <=0;
    end
    else if(cnt<17'd70000  &&  cnt>=17'd60000&&key_pulse[1])
	begin
		 counter_B_6 <= counter_B_6 + 1;
         counter_B_6_reg <= counter_B_6;
		 counter_B_5 <=0;
	end
    else if(cnt<17'd80000 &&  cnt>=17'd70000&&key_pulse[1])
	begin
		 counter_B_7 <= counter_B_7 + 1;
		 counter_B_7_reg <= counter_B_7;
		 counter_B_6 <=0;
	end
    else if(cnt<17'd90000 &&  cnt>=17'd80000&&key_pulse[1])
	begin
		 counter_B_8 <= counter_B_8 + 1;
		 counter_B_8_reg <= counter_B_8;
		 counter_B_7 <=0;
	end
    else if(cnt<17'd100000&&key_pulse[1]&&cnt>17'd90000)
	begin
		 counter_B_9 <= counter_B_9 + 1;
		 counter_B_9_reg <= counter_B_9;
		 counter_B_8 <=0;	
    end

    end

assign counter_A =  counter_A_0_reg+counter_A_1_reg+counter_A_2_reg+counter_A_3_reg+counter_A_4_reg+counter_A_5_reg+counter_A_6_reg+counter_A_7_reg+counter_A_8_reg+counter_A_9_reg;
assign counter_B =  0;// counter_B_0_reg+counter_B_1_reg+counter_B_2_reg+counter_B_3_reg+counter_B_4_reg+counter_B_5_reg+counter_B_6_reg+counter_B_7_reg+counter_B_8_reg+counter_B_9_reg;
     

endmodule : encoder_core
